"""The mix-in class implementation for the `get_bounds` method.
"""

from typing import Optional

from typing_extensions import final

from apysc._display.display_object import DisplayObject
from apysc._geom.rectangle_geom import RectangleGeom
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.int import Int
from apysc._type.number import Number
from apysc._type.variable_name_mixin import VariableNameMixIn


class GetBoundsMixIn(VariableNameMixIn):
    @final
    @add_debug_info_setting(module_name=__name__)
    def get_bounds(
        self,
        target_coordinate_space_object: Optional[DisplayObject] = None,
    ) -> RectangleGeom:
        """
        Get an instance's bounding-box geometry data.

        Parameters
        ----------
        target_coordinate_space_object : DisplayObject or None, default None
            Target coordinate space object. If None is specified, then this
            method returns the bounding-box data based on the stage.
            If a `DisplayObject` instance is specified, then this method
            returns the bounding-box data based on the specified object.

        Returns
        -------
        bounding_box : RectangleGeom
            An instance's bounding-box geometry data.

        References
        ----------
        - get_bounds interface
            - https://simon-ritchie.github.io/apysc/en/get_bounds.md
        - RectangleGeom class
            - https://simon-ritchie.github.io/apysc/en/rectangle_geom.html

        Examples
        --------
        >>> import apysc as ap

        >>> stage: ap.Stage = ap.Stage(
        ...     background_color=ap.Color("#333"), stage_width=250, stage_height=350
        ... )
        >>> rectangle: ap.Rectangle = ap.Rectangle(
        ...     x=50,
        ...     y=100,
        ...     width=150,
        ...     height=200,
        ...     fill_color=ap.Color("#0af"),
        ... )
        >>> bounding_box: ap.RectangleGeom = rectangle.get_bounds()
        """
        from apysc._type.variable_name_suffix_utils import (
            get_attr_or_variable_name_suffix,
        )

        suffix: str = get_attr_or_variable_name_suffix(
            instance=self, value_identifier="left_x"
        )
        left_x: Number = Number(0, variable_name_suffix=suffix)

        suffix = get_attr_or_variable_name_suffix(
            instance=self, value_identifier="center_x"
        )
        center_x: Number = Number(0, variable_name_suffix=suffix)

        suffix = get_attr_or_variable_name_suffix(
            instance=self, value_identifier="right_x"
        )
        right_x: Number = Number(0, variable_name_suffix=suffix)

        suffix = get_attr_or_variable_name_suffix(
            instance=self, value_identifier="top_y"
        )
        top_y: Number = Number(0, variable_name_suffix=suffix)

        suffix = get_attr_or_variable_name_suffix(
            instance=self, value_identifier="center_y"
        )
        center_y: Number = Number(0, variable_name_suffix=suffix)

        suffix = get_attr_or_variable_name_suffix(
            instance=self, value_identifier="bottom_y"
        )
        bottom_y: Number = Number(0, variable_name_suffix=suffix)

        suffix = get_attr_or_variable_name_suffix(
            instance=self, value_identifier="width"
        )
        width: Int = Int(0, variable_name_suffix=suffix)

        suffix = get_attr_or_variable_name_suffix(
            instance=self, value_identifier="height"
        )
        height: Int = Int(0, variable_name_suffix=suffix)
        self._append_get_bounds_expression(
            target_coordinate_space_object=target_coordinate_space_object,
            left_x=left_x,
            center_x=center_x,
            right_x=right_x,
            top_y=top_y,
            center_y=center_y,
            bottom_y=bottom_y,
            width=width,
            height=height,
        )
        bounding_box: RectangleGeom = RectangleGeom(
            left_x=left_x,
            center_x=center_x,
            right_x=right_x,
            top_y=top_y,
            center_y=center_y,
            bottom_y=bottom_y,
            width=width,
            height=height,
        )
        return bounding_box

    def _append_get_bounds_expression(
        self,
        *,
        target_coordinate_space_object: Optional[DisplayObject],
        left_x: Number,
        center_x: Number,
        right_x: Number,
        top_y: Number,
        center_y: Number,
        bottom_y: Number,
        width: Int,
        height: Int,
    ) -> None:
        """
        Append a `get_counds` method expression string.

        Parameters
        ----------
        target_coordinate_space_object : DisplayObject or None, default None
            Target coordinate space object.
        left_x : Number
            The rectangle left x coordinate.
        center_x : Number
            The rectangle center x coordinate.
        right_x : Number
            The rectangle right x coordinate.
        top_y : Number
            The rectangle top y coordinate.
        center_y : Number
            The rectangle center y coordinate.
        bottom_y : Number
            The rectangle bottom y coordinate.
        width : Int
            The rectangle width.
        height : Int
            The Rectangle height.
        """
        from apysc._display.stage import Stage
        from apysc._display.stage import get_stage
        from apysc._expression import expression_data_util

        if target_coordinate_space_object is None:
            stage: Stage = get_stage()
            coordinate_space_object_variable_name: str = stage.variable_name
        else:
            coordinate_space_object_variable_name = (
                target_coordinate_space_object.variable_name
            )
        expression: str = (
            f"var box = {self.variable_name}"
            f".rbox({coordinate_space_object_variable_name});"
            f"\n{left_x.variable_name} = box.x;"
            f"\n{center_x.variable_name} = box.cx;"
            f"\n{right_x.variable_name} = box.x2;"
            f"\n{top_y.variable_name} = box.y;"
            f"\n{center_y.variable_name} = box.cy;"
            f"\n{bottom_y.variable_name} = box.y2;"
            f"\n{width.variable_name} = box.width;"
            f"\n{height.variable_name} = box.height;"
        )
        expression_data_util.append_js_expression(expression=expression)



"""The class implementation for the outlined `phone_enabled` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialPhoneEnabledOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `phone_enabled` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><rect fill="none" height="24" width="24"/><g><g><path d="M17.46,5c-0.06,0.89-0.21,1.76-0.45,2.59l1.2,1.2c0.41-1.2,0.67-2.47,0.76-3.79H17.46z M7.6,17.02 c-0.85,0.24-1.72,0.39-2.6,0.45v1.49c1.32-0.09,2.59-0.35,3.8-0.75L7.6,17.02z M16.5,3H20c0.55,0,1,0.45,1,1c0,9.39-7.61,17-17,17 c-0.55,0-1-0.45-1-1v-3.49c0-0.55,0.45-1,1-1c1.24,0,2.45-0.2,3.57-0.57c0.1-0.04,0.21-0.05,0.31-0.05c0.26,0,0.51,0.1,0.71,0.29 l2.2,2.2c2.83-1.45,5.15-3.76,6.59-6.59l-2.2-2.2c-0.28-0.28-0.36-0.67-0.25-1.02C15.3,6.45,15.5,5.25,15.5,4 C15.5,3.45,15.95,3,16.5,3z"/></g></g></svg>'  # noqa



"""This module is for the translation mapping data of the
following document:

Document file: text_italic.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Text italic property": "# テキストの italic 属性",
    ##################################################
    "This page explains the text-related `italic` property.": "このページではテキスト関係の`italic`属性について説明します。",  # noqa
    ##################################################
    "## What property is this?": "## 属性の概要",
    ##################################################
    "The `italic` property updates or gets the instance's italic style setting.": "`italic`属性はインスタンスの斜体のスタイル設定の更新もしくは取得を行うことができます。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The getter and setter interfaces' type becomes an `ap.Boolean` value.": "getterもしくはsetterの各インターフェイスの値は`ap.Boolean`の型となります。",  # noqa
    ##################################################
    "If you specify the `ap.Boolean(True)` (or `ap.True_`) value, a text instance becomes italic style.": "もし`ap.Boolean(True)`（もしくは`ap.True_`）の値を指定した場合、テキストインスタンスは斜体のスタイル設定になります。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=350,\n    stage_height=170,\n    stage_elem_id="stage",\n)\nmulti_line_text: ap.MultiLineText = ap.MultiLineText(\n    text="Lorem ipsum dolor sit amet, consectetur adipiscing elit, "\n    "sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. "\n    "Ut enim ad minim veniam",\n    width=300,\n    font_size=16,\n    fill_color=ap.Color("#00aaff"),\n    x=25,\n    y=25,\n)\nmulti_line_text.italic = ap.True_\n\nap.save_overall_html(dest_dir_path="./text_italic_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=350,\n    stage_height=170,\n    stage_elem_id="stage",\n)\nmulti_line_text: ap.MultiLineText = ap.MultiLineText(\n    text="Lorem ipsum dolor sit amet, consectetur adipiscing elit, "\n    "sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. "\n    "Ut enim ad minim veniam",\n    width=300,\n    font_size=16,\n    fill_color=ap.Color("#00aaff"),\n    x=25,\n    y=25,\n)\nmulti_line_text.italic = ap.True_\n\nap.save_overall_html(dest_dir_path="./text_italic_basic_usage/")\n```',  # noqa
    ##################################################
    "## italic property API": "## italic 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get an italic (font-style) value.<hr>": "斜体のスタイル（font-style）設定を取得します。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `italic`: Boolean": "- `italic`: Boolean",
    ##################################################
    "  - An italic (font-style) value.": "  - 斜体のスタイル（font-style）の設定値。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage(\n...     background_color=ap.Color("#333"),\n...     stage_width=350,\n...     stage_height=170,\n...     stage_elem_id="stage",\n... )\n>>> multi_line_text: ap.MultiLineText = ap.MultiLineText(\n...     text="Lorem ipsum dolor sit amet, consectetur adipiscing elit, "\n...     "sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. "\n...     "Ut enim ad minim veniam",\n...     width=300,\n...     font_size=16,\n...     fill_color=ap.Color("#00aaff"),\n...     x=25,\n...     y=25,\n... )\n>>> multi_line_text.italic = ap.True_\n>>> multi_line_text.italic\nBoolean(True)\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage(\n...     background_color=ap.Color("#333"),\n...     stage_width=350,\n...     stage_height=170,\n...     stage_elem_id="stage",\n... )\n>>> multi_line_text: ap.MultiLineText = ap.MultiLineText(\n...     text="Lorem ipsum dolor sit amet, consectetur adipiscing elit, "\n...     "sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. "\n...     "Ut enim ad minim veniam",\n...     width=300,\n...     font_size=16,\n...     fill_color=ap.Color("#00aaff"),\n...     x=25,\n...     y=25,\n... )\n>>> multi_line_text.italic = ap.True_\n>>> multi_line_text.italic\nBoolean(True)\n```',  # noqa
}



"""The mix-in class implementation for the single column's
`_create_y_axis` method.
"""

from typing import Optional
from typing import Tuple
from typing import Union
from typing import cast

from typing_extensions import final

from apysc._chart import chart_const
from apysc._chart.x_axis_settings import XAxisSettings
from apysc._chart.y_axis_single_column_settings import YAxisSingleColumnSettings
from apysc._color.color import Color
from apysc._display.line import Line
from apysc._display.sprite import Sprite
from apysc._display.svg_text import SvgText
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.array import Array
from apysc._type.boolean import Boolean
from apysc._type.dictionary import Dictionary
from apysc._type.int import Int
from apysc._type.number import Number
from apysc._type.string import String


class CreateSingleColumnYAxisMixIn:
    _in_value_y_min: Number
    _in_value_y_max: Number
    _y_axis_height: Int
    _y_axis_ticks_num: Int
    _y_axis_min: Number
    _y_axis_max: Number
    _y_axis_text_container: Sprite
    _y_axis_ticks_y_coordinates: Array[Number]
    _y_axis_texts_values: Array[String]
    _y_axis_ticks_texts: Array[SvgText]
    _y_axis_texts_container: Sprite
    _y_axis_border: Line

    @final
    @add_debug_info_setting(module_name=__name__)
    def _create_y_axis(
        self,
        *,
        data: Array[Dictionary[String, Union[Int, Number, String]]],
        y_axis_container: Sprite,
        chart_height: Int,
        x_axis_settings: XAxisSettings,
        y_axis_settings: YAxisSingleColumnSettings,
        vertical_padding: Int,
        horizontal_padding: Int,
        variable_name_suffix: str,
    ) -> None:
        """
        Create a y-axis instance.

        Parameters
        ----------
        data : Array[Dictionary[String, Union[Int, Number, String]]],
            A data array, which contains a 1-dimensional string key dictionary.
        y_axis_container : Sprite
            A y-axis container instance.
        chart_height : Int
            A chart height.
        x_axis_settings : XAxisSettings
            An x-axis settings instance.
        y_axis_settings : YAxisSingleColumnSettings
            A y-axis settings instance.
        vertical_padding : Int
            A chart's vertical padding between borders and contents.
        horizontal_padding : Int
            A chart's horizontal padding between borders and contents.
        variable_name_suffix : str, optional
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.
        """
        self._in_value_y_min = _calculate_y_min_from_data(
            data=data,
            y_axis_column_name=y_axis_settings._y_axis_column_name,
            variable_name_suffix=variable_name_suffix,
        )
        self._in_value_y_max = _calculate_y_max_from_data(
            data=data,
            y_axis_column_name=y_axis_settings._y_axis_column_name,
            variable_name_suffix=variable_name_suffix,
        )
        self._y_axis_height = _calculate_y_axis_height(
            chart_height=chart_height,
            vertical_padding=vertical_padding,
            tick_text_font_size=x_axis_settings._tick_text_font_size,
            axis_label_font_size=x_axis_settings._axis_label_font_size,
            is_display_axis_label=x_axis_settings._is_display_axis_label,
            variable_name_suffix=variable_name_suffix,
        )
        self._y_axis_ticks_num = _calculate_y_axis_ticks_num(
            y_axis_height=self._y_axis_height,
            tick_max_num=y_axis_settings._tick_max_num,
            tick_text_font_size=y_axis_settings._tick_text_font_size,
            variable_name_suffix=variable_name_suffix,
        )
        self._y_axis_ticks_y_coordinates = _calculate_y_axis_ticks_y_coordinates(
            vertical_padding=vertical_padding,
            y_axis_height=self._y_axis_height,
            y_axis_ticks_num=self._y_axis_ticks_num,
            font_size=y_axis_settings._tick_text_font_size,
            variable_name_suffix=variable_name_suffix,
        )
        self._y_axis_min = _calculate_y_axis_min(
            y_min=y_axis_settings._y_min,
            in_value_y_min=self._in_value_y_min,
        )
        self._y_axis_max = _calculate_y_axis_max(
            y_max=y_axis_settings._y_max,
            in_value_y_max=self._in_value_y_max,
        )
        self._y_axis_texts_values = _create_y_axis_texts_values(
            y_axis_min=self._y_axis_min,
            y_axis_max=self._y_axis_max,
            ticks_num=self._y_axis_ticks_y_coordinates.length,
            max_num_of_decimal_places=(
                y_axis_settings._tick_text_max_num_of_decimal_places
            ),
            variable_name_suffix=variable_name_suffix,
        )
        (
            self._y_axis_ticks_texts,
            self._y_axis_texts_container,
        ) = _create_y_axis_ticks_texts(
            y_axis_container=y_axis_container,
            horizontal_padding=horizontal_padding,
            y_axis_text_values=self._y_axis_texts_values,
            y_axis_ticks_y_coordinates=self._y_axis_ticks_y_coordinates,
            tick_text_fill_color=y_axis_settings._tick_text_fill_color,
            tick_text_fill_alpha=y_axis_settings._tick_text_fill_alpha,
            tick_text_font_size=y_axis_settings._tick_text_font_size,
            tick_text_font_family=y_axis_settings._tick_text_font_family,
            tick_text_bold=y_axis_settings._tick_text_bold,
            tick_text_italic=y_axis_settings._tick_text_bold,
            variable_name_suffix=variable_name_suffix,
        )
        self._y_axis_border = _create_y_axis_vertical_border(
            y_axis_container=y_axis_container,
            y_axis_texts_container=self._y_axis_texts_container,
            line_color=y_axis_settings._line_color,
            line_thickness=y_axis_settings._line_thickness,
            line_alpha=y_axis_settings._line_alpha,
            tick_text_font_size=y_axis_settings._tick_text_font_size,
            variable_name_suffix=variable_name_suffix,
        )


def _create_y_axis_vertical_border(
    *,
    y_axis_container: Sprite,
    y_axis_texts_container: Sprite,
    line_color: Color,
    line_thickness: Int,
    line_alpha: Number,
    tick_text_font_size: Int,
    variable_name_suffix: str,
) -> Line:
    """
    Create a y-axis vertical border.

    Parameters
    ----------
    y_axis_container : Sprite
        A y-axis container instance.
    y_axis_texts_container : Sprite
        A y-axis texts container instance.
    line_color : Color
        A line color setting.
    line_thickness : Int
        A line thickness setting.
    line_alpha : Number
        A line alpha setting.
    tick_text_font_size : Int
        A tick text font size setting.
    variable_name_suffix : str, optional
        A JavaScript variable name suffix string.
        This setting is sometimes useful for JavaScript debugging.

    Returns
    -------
    line : Line
        A created line.
    """
    from apysc._geom.point2d import Point2D
    from apysc._geom.rectangle_geom import RectangleGeom

    bounding_box: RectangleGeom = y_axis_texts_container.get_bounds()
    x: Number = bounding_box.width.to_number() + chart_const.LARGE_PADDING * 3
    y_start: Number = Number(
        chart_const.SMALL_PADDING,
        variable_name_suffix=variable_name_suffix,
    )
    y_end: Number = bounding_box.height.to_number() + y_start - tick_text_font_size / 2
    line: Line = Line(
        start_point=Point2D(x=x, y=y_start),
        end_point=Point2D(x=x, y=y_end),
        line_color=line_color,
        line_thickness=line_thickness,
        line_alpha=line_alpha,
        parent=y_axis_container,
        variable_name_suffix=variable_name_suffix,
    )
    return line


def _create_y_axis_ticks_texts(
    *,
    y_axis_container: Sprite,
    horizontal_padding: Int,
    y_axis_text_values: Array[String],
    y_axis_ticks_y_coordinates: Array[Number],
    tick_text_fill_color: Color,
    tick_text_fill_alpha: Number,
    tick_text_font_size: Int,
    tick_text_font_family: Optional[Array[String]],
    tick_text_bold: Boolean,
    tick_text_italic: Boolean,
    variable_name_suffix: str,
) -> Tuple[Array[SvgText], Sprite]:
    """
    Create a y-axis ticks texts.

    Parameters
    ----------
    y_axis_container : Sprite
        A y-axis container instance.
    horizontal_padding : Int
        A chart horizontal padding.
    y_axis_text_values : Array[String]
        A y-axis text values.
    y_axis_ticks_y_coordinates : Array[Number]
        A y-axis ticks y coordinates.
    tick_text_fill_color : Color
        A tick text fill-color.
    tick_text_fill_alpha : Number
        A tick text fill-alpha.
    tick_text_font_size : Int
        A tick text font size.
    tick_text_font_family : Optional[Array[String]]
        A tick text font family.
    tick_text_bold : Boolean
        A boolean indicates whether a tick text is bold or not.
    tick_text_italic : Boolean
        A boolean indicates whether a tick text is italic or not.
    variable_name_suffix : str
        A JavaScript variable name suffix string.
        This setting is sometimes useful for JavaScript debugging.

    Returns
    -------
    y_axis_ticks_texts : Array[SvgText]
        Created y-axis ticks texts.
    texts_container : Sprite
        A container that contains created texts.
    """
    from apysc._display.svg_text_align_mixin import SvgTextAlign
    from apysc._loop.for_array_indices import ForArrayIndices

    y_axis_ticks_texts: Array[SvgText] = Array([])
    texts_container: Sprite = Sprite()
    y_axis_container.add_child(texts_container)
    with ForArrayIndices(arr=y_axis_ticks_y_coordinates) as i:
        txt: SvgText = SvgText(
            text=y_axis_text_values[i],
            font_size=tick_text_font_size,
            font_family=tick_text_font_family,
            x=0,
            y=y_axis_ticks_y_coordinates[i],
            fill_color=tick_text_fill_color,
            fill_alpha=tick_text_fill_alpha,
            align=SvgTextAlign.RIGHT,
            bold=tick_text_bold,
            italic=tick_text_italic,
            parent=texts_container,
            variable_name_suffix=variable_name_suffix,
        )
        y_axis_ticks_texts.append(txt)
    _apply_x_coordinate_to_y_axis_ticks_texts(
        horizontal_padding=horizontal_padding,
        y_axis_ticks_texts=y_axis_ticks_texts,
        x_coordinate_container=texts_container,
        variable_name_suffix=variable_name_suffix,
    )
    return y_axis_ticks_texts, texts_container


def _create_y_axis_texts_values(
    *,
    y_axis_min: Number,
    y_axis_max: Number,
    ticks_num: Int,
    max_num_of_decimal_places: Int,
    variable_name_suffix: str,
) -> Array[String]:
    """
    Create y-axis texts values.

    Parameters
    ----------
    y_axis_min : Number
        A y-axis min value.
    y_axis_max : Number
        A y-axis max value.
    ticks_num : Int
        A ticks number.
    max_num_of_decimal_places : Int
        A maximum number of decimal places.
    variable_name_suffix : str
        A JavaScript variable name suffix string.
        This setting is sometimes useful for JavaScript debugging.

    Returns
    -------
    y_axis_text_values : Array[String]
        A created y-axis texts values.
    """
    from apysc._loop._range import range as ap_range
    from apysc._loop.for_array_indices import ForArrayIndices

    diff: Number = y_axis_max - y_axis_min
    interval: Number = diff / (ticks_num - 1)
    y_axis_text_values: Array[String] = Array(
        [], variable_name_suffix=variable_name_suffix
    )
    range_arr: Array[Int] = ap_range(ticks_num)
    with ForArrayIndices(arr=range_arr) as i:
        tick_value: Union[Int, Number] = y_axis_min + i * interval
        value_text: String = cast(Union[Int, Number], tick_value).to_string()
        value_text = value_text.apply_max_num_of_decimal_places(
            max_num_of_decimal_places=max_num_of_decimal_places
        )
        y_axis_text_values.append(value_text)
    return y_axis_text_values


def _calculate_y_axis_min(
    *,
    y_min: Optional[Number],
    in_value_y_min: Number,
) -> Number:
    """
    Calculate a y-axis min value.

    Parameters
    ----------
    y_min : Optional[Number]
        A y-axis min setting.
    in_value_y_min : Number
        A minimum y value in data.

    Returns
    -------
    y_axis_min : Number
        A calculated y-axis min value.
    """
    if y_min is not None:
        return y_min.copy()
    return in_value_y_min


def _calculate_y_axis_max(
    *,
    y_max: Optional[Number],
    in_value_y_max: Number,
) -> Number:
    """
    Calculate a y-axis max value.

    Parameters
    ----------
    y_max : Optional[Number]
        A y-axis max setting (limitation).
    in_value_y_max : Number
        A maximum y value in data.

    Returns
    -------
    y_axis_max : Number
        A calculated y-axis max value.
    """
    if y_max is not None:
        return y_max.copy()
    y_axis_max: Number = in_value_y_max * 1.1
    return y_axis_max


def _apply_x_coordinate_to_y_axis_ticks_texts(
    *,
    horizontal_padding: Int,
    y_axis_ticks_texts: Array[SvgText],
    x_coordinate_container: Sprite,
    variable_name_suffix: str,
) -> None:
    """
    Apply x-coordinate to y-axis ticks texts.

    Parameters
    ----------
    horizontal_padding : Int
        A chart horizontal padding.
    y_axis_ticks_texts : Array[SvgText]
        Y-axis ticks' texts.
    x_coordinate_container : Sprite
        A ticks container.
    variable_name_suffix : str
        A JavaScript variable name suffix string.
        This setting is sometimes useful for JavaScript debugging.
    """
    from apysc._display.svg_text import SvgText
    from apysc._geom.rectangle_geom import RectangleGeom
    from apysc._loop.for_array_indices import ForArrayIndices
    from apysc._math.math import Math
    from apysc._type.array import Array

    x: Number = horizontal_padding.copy().to_number()
    max_arr: Array[Union[Int, Number]] = Array(
        [x],
        variable_name_suffix=variable_name_suffix,
    )
    with ForArrayIndices(arr=y_axis_ticks_texts) as i:
        txt: SvgText = y_axis_ticks_texts[i]
        bounding_box: RectangleGeom = txt.get_bounds()
        max_arr.append(bounding_box.width + horizontal_padding)
    x = Math.max(max_arr)
    x_coordinate_container.x = x


def _calculate_y_axis_ticks_y_coordinates(
    *,
    vertical_padding: Int,
    y_axis_height: Int,
    y_axis_ticks_num: Int,
    font_size: Int,
    variable_name_suffix: str,
) -> Array[Number]:
    """
    Calculate y-axis ticks coordinates.

    Parameters
    ----------
    vertical_padding : Int
        A chart's vertical padding between borders and contents.
    y_axis_height : Int
        An axis height.
    y_axis_ticks_num : Int
        Axis tick number.
    font_size : Int
        A ticks text font size.
    variable_name_suffix : str
        A JavaScript variable name suffix string.
        This setting is sometimes useful for JavaScript debugging.

    Returns
    -------
    y_axis_ticks_y_coordinates : Array[Number]
        Y-axis ticks coordinates. The first index becomes the axis
        starting coordinate (bottom position of a y-axis).
    """
    from apysc._loop._range import range as ap_range
    from apysc._loop.for_array_indices import ForArrayIndices

    y_axis_ticks_y_coordinates: Array[Number] = Array(
        [],
        variable_name_suffix=variable_name_suffix,
        fixed_value_type=Number,
    )
    y_start_coordinate: Int = Int(
        vertical_padding + font_size,
        variable_name_suffix=variable_name_suffix,
    )
    range_arr: Array[Int] = ap_range(y_axis_ticks_num)
    interval: Number = (y_axis_height - font_size) / (y_axis_ticks_num - 1)
    with ForArrayIndices(range_arr) as i:
        y_coordinate: Number = interval * i + y_start_coordinate
        y_axis_ticks_y_coordinates.append(y_coordinate)
    y_axis_ticks_y_coordinates.reverse()
    return y_axis_ticks_y_coordinates


def _calculate_y_axis_ticks_num(
    *,
    y_axis_height: Int,
    tick_max_num: Optional[Int],
    tick_text_font_size: Int,
    variable_name_suffix: str,
) -> Int:
    """
    Calculate y-axis ticks number.

    Parameters
    ----------
    y_axis_height : Int
        A y-axis height.
    tick_max_num : Optional[Int]
        A maximum ticks number.
    tick_text_font_size : Int
        A tick text font size.
    variable_name_suffix : str, optional
        A JavaScript variable name suffix string.
        This setting is sometimes useful for JavaScript debugging.

    Returns
    -------
    y_axis_ticks_num : Int
        Y-axis ticks number.
    """
    from apysc._math.math import Math
    from apysc._type.array import Array

    interval: Int = tick_text_font_size * 3
    y_axis_ticks_num: Int = y_axis_height // interval
    y_axis_ticks_num += 1
    if tick_max_num is not None:
        min_arr: Array[Int] = Array(
            [y_axis_ticks_num, tick_max_num],
            variable_name_suffix=variable_name_suffix,
            fixed_value_type=Int,
        )
        y_axis_ticks_num = Int(
            Math.min(min_arr), variable_name_suffix=variable_name_suffix
        )
    return y_axis_ticks_num


def _calculate_y_axis_height(
    *,
    chart_height: Int,
    vertical_padding: Int,
    tick_text_font_size: Int,
    axis_label_font_size: Int,
    is_display_axis_label: Boolean,
    variable_name_suffix: str,
) -> Int:
    """
    Calculate a y-axis height.

    Parameters
    ----------
    chart_height : Int
        A chart height.
    vertical_padding : Int
        A chart's vertical padding between borders and contents.
    tick_text_font_size : Int
        A tick text font size.
    axis_label_font_size : Int
        An axis label font size.
    is_display_axis_label : Boolean
        A boolean, whether an axis label is visible or not.
    variable_name_suffix : str, optional
        A JavaScript variable name suffix string.
        This setting is sometimes useful for JavaScript debugging.

    Returns
    -------
    y_axis_height : Int
        Calculate height.
    """
    from apysc._branch._if import If

    y_axis_height: Int = Int(chart_height, variable_name_suffix=variable_name_suffix)
    y_axis_height -= vertical_padding * 2
    y_axis_height -= axis_label_font_size
    y_axis_height -= chart_const.SMALL_PADDING
    y_axis_height -= chart_const.TICK_SIZE
    with If(is_display_axis_label):
        y_axis_height -= chart_const.SMALL_PADDING * 2
        y_axis_height -= tick_text_font_size
    return y_axis_height


def _calculate_y_max_from_data(
    *,
    data: Array[Dictionary[String, Union[Int, Number, String]]],
    y_axis_column_name: String,
    variable_name_suffix: str,
) -> Number:
    """
    Get a maximum y-axis value from specified data.

    Parameters
    ----------
    data : Array[Dictionary[String, Union[Int, Number, String]]]
        A data array, which contains a 1-dimensional string key dictionary.
    y_axis_column_name : String
        A y-axis column name.
    variable_name_suffix : str
        A JavaScript variable name suffix string.
        This setting is sometimes useful for JavaScript debugging.

    Returns
    -------
    y_max : Number
        A maximum y-axis value.
    """
    from apysc._math.math import Math

    values: Array[Union[Int, Number]] = _extract_column_values_from_data(
        data=data,
        column_name=y_axis_column_name,
        variable_name_suffix=variable_name_suffix,
    )
    y_max: Number = Math.max(values)
    return y_max


def _calculate_y_min_from_data(
    *,
    data: Array[Dictionary[String, Union[Int, Number, String]]],
    y_axis_column_name: String,
    variable_name_suffix: str,
) -> Number:
    """
    Get a minimum y-axis value from specified data.

    Parameters
    ----------
    data : Array[Dictionary[String, Union[Int, Number, String]]]
        A data array, which contains a 1-dimensional string key dictionary.
    y_axis_column_name : String
        A y-axis column name.
    variable_name_suffix : str, optional
        A JavaScript variable name suffix string.
        This setting is sometimes useful for JavaScript debugging.

    Returns
    -------
    y_min : Number
        A minimum y-axis value.
    """
    from apysc._math.math import Math

    values: Array[Union[Int, Number]] = _extract_column_values_from_data(
        data=data,
        column_name=y_axis_column_name,
        variable_name_suffix=variable_name_suffix,
    )
    y_min: Number = Math.min(values)
    return y_min


def _extract_column_values_from_data(
    *,
    data: Array[Dictionary[String, Union[Int, Number, String]]],
    column_name: String,
    variable_name_suffix: str,
) -> Array[Union[Int, Number]]:
    """
    Get a specified column values array from a specified data.

    Parameters
    ----------
    data : Array[Dictionary[String, Union[Int, Number, String]]]
        A data array, which contains a 1-dimensional string key dictionary.
    column_name : String
        A y-axis column name.
    variable_name_suffix : str, optional
        A JavaScript variable name suffix string.
        This setting is sometimes useful for JavaScript debugging.

    Returns
    -------
    values : Array[Union[Int, Number]]
        A values array.
    """
    from apysc._loop.for_array_indices import ForArrayIndices
    from apysc._type.array import Array

    values: Array[Union[Int, Number]] = Array(
        [],
        variable_name_suffix=variable_name_suffix,
        fixed_value_type=Number,
    )
    with ForArrayIndices(data) as i:
        dict_value: Dictionary[String, Union[Int, Number]] = cast(
            Dictionary[String, Union[Int, Number]], data[i]
        )
        values.append(dict_value[column_name])
    return values



"""The class implementation for the `visibility_off` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialVisibilityOffIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `visibility_off` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0zm0 0h24v24H0zm0 0h24v24H0zm0 0h24v24H0z" fill="none"/><path d="M12 7c2.76 0 5 2.24 5 5 0 .65-.13 1.26-.36 1.83l2.92 2.92c1.51-1.26 2.7-2.89 3.43-4.75-1.73-4.39-6-7.5-11-7.5-1.4 0-2.74.25-3.98.7l2.16 2.16C10.74 7.13 11.35 7 12 7zM2 4.27l2.28 2.28.46.46C3.08 8.3 1.78 10.02 1 12c1.73 4.39 6 7.5 11 7.5 1.55 0 3.03-.3 4.38-.84l.42.42L19.73 22 21 20.73 3.27 3 2 4.27zM7.53 9.8l1.55 1.55c-.05.21-.08.43-.08.65 0 1.66 1.34 3 3 3 .22 0 .44-.03.65-.08l1.55 1.55c-.67.33-1.41.53-2.2.53-2.76 0-5-2.24-5-5 0-.79.2-1.53.53-2.2zm4.31-.78l3.15 3.15.02-.16c0-1.66-1.34-3-3-3l-.17.01z"/></svg>'  # noqa



"""The top y geometry mix-in implementation for the `RectangleGeom`.
"""

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.number import Number


class RectangleGeomTopYMixIn:
    _top_y: Number

    @property
    @add_debug_info_setting(module_name=__name__)
    def top_y(self) -> Number:
        """
        Get the rectangle top y coordinate.

        Returns
        -------
        top_y : Number
            The rectangle top y coordinate.
        """
        return self._top_y



"""Class implementation for the continue.
"""

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting


class Continue:
    """
    The loop continue expression class.

    Notes
    -----
    This class can be instantiated in the with loop statement,
    for example, after the `with ap.ForArrayIndices(...):` statement.

    References
    ----------
    - Continue
        - https://simon-ritchie.github.io/apysc/en/continue.html

    Examples
    --------
    >>> import apysc as ap
    >>> arr: ap.Array = ap.Array(range(3))
    >>> with ap.ForArrayIndices(arr) as i:
    ...     with ap.If(i == 1):
    ...         _ = ap.Continue()
    ...
    """

    @final
    @add_debug_info_setting(module_name=__name__)
    def __init__(self) -> None:
        """
        The loop continue expression class.

        Notes
        -----
        This class can be instantiated in the with loop statement,
        for example, after the `with ap.ForArrayIndices(...):` statement.

        References
        ----------
        - Continue
            - https://simon-ritchie.github.io/apysc/en/continue.html

        Examples
        --------
        >>> import apysc as ap
        >>> arr: ap.Array = ap.Array(range(3))
        >>> with ap.ForArrayIndices(arr) as i:
        ...     with ap.If(i == 1):
        ...         _ = ap.Continue()
        ...
        """
        from apysc._expression import expression_data_util
        from apysc._loop import loop_count

        current_loop_count: int = loop_count.get_current_loop_count()
        if current_loop_count == 0:
            err_msg: str = (
                "The `Continue` class can be instantiated in the with "
                "loop statement, for example, after the "
                "`with ap.ForArrayIndices(...):` statement."
            )
            raise Exception(err_msg)
        expression_data_util.append_js_expression(expression="continue;")



"""The class implementation for the `settings_input_composite` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialSettingsInputCompositeIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `settings_input_composite` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0z" fill="none"/><path d="M5 2c0-.55-.45-1-1-1s-1 .45-1 1v4H1v6h6V6H5V2zm4 14c0 1.3.84 2.4 2 2.82V23h2v-4.18c1.16-.41 2-1.51 2-2.82v-2H9v2zm-8 0c0 1.3.84 2.4 2 2.82V23h2v-4.18C6.16 18.4 7 17.3 7 16v-2H1v2zM21 6V2c0-.55-.45-1-1-1s-1 .45-1 1v4h-2v6h6V6h-2zm-8-4c0-.55-.45-1-1-1s-1 .45-1 1v4H9v6h6V6h-2V2zm4 14c0 1.3.84 2.4 2 2.82V23h2v-4.18c1.16-.41 2-1.51 2-2.82v-2h-6v2z"/></svg>'  # noqa



"""The class implementation for the `swipe` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialSwipeIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `swipe` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M19.75 16.25c0 .06-.01.13-.02.2l-.75 5.27c-.11.73-.69 1.28-1.44 1.28h-6.79c-.41 0-.79-.17-1.06-.44l-4.94-4.94.79-.8c.2-.2.48-.33.79-.33.09 0 .16.02.24.03l3.43.72V6.5c0-.83.67-1.5 1.5-1.5s1.5.67 1.5 1.5v6h.76c.19 0 .37.04.54.11l4.54 2.26c.53.22.91.76.91 1.38zm.38-12.38C18.69 2.17 15.6 1 12 1S5.31 2.17 3.87 3.87L2 2v5h5L4.93 4.93c1-1.29 3.7-2.43 7.07-2.43s6.07 1.14 7.07 2.43L17 7h5V2l-1.87 1.87z"/></svg>'  # noqa



"""Class implementation for the rotation_around_point mix-in.
"""

from typing import Any
from typing import Dict
from typing import Optional

from typing_extensions import final

from apysc._animation.animation_rotation_around_point_mixin import (
    AnimationRotationAroundPointMixIn,
)
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.dictionary import Dictionary
from apysc._type.int import Int
from apysc._type.number import Number
from apysc._type.revert_mixin import RevertMixIn
from apysc._type.variable_name_suffix_attr_or_var_mixin import (
    VariableNameSuffixAttrOrVarMixIn,
)
from apysc._validation import arg_validation_decos


class RotationAroundPointMixIn(
    VariableNameSuffixAttrOrVarMixIn,
    AnimationRotationAroundPointMixIn,
    RevertMixIn,
):
    _rotation_around_point: Dictionary[str, Int]

    @final
    def _initialize_rotation_around_point_if_not_initialized(self) -> None:
        """
        Initialize the `_rotation_around_point` attribute
        if this interface does not initialize it yet.
        """
        if hasattr(self, "_rotation_around_point"):
            return
        suffix: str = self._get_attr_or_variable_name_suffix(
            value_identifier="rotation_around_point"
        )
        self._rotation_around_point = Dictionary(
            {},
            variable_name_suffix=suffix,
            skip_init_substitution_expression_appending=True,
        )

    @final
    @arg_validation_decos.is_num(arg_position_index=1, optional=False)
    @arg_validation_decos.is_num(arg_position_index=2, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def get_rotation_around_point(self, *, x: Number, y: Number) -> Int:
        """
        Get a rotation value around the given coordinates.

        Parameters
        ----------
        x : Number
            X-coordinate.
        y : Number
            Y-coordinate.

        Returns
        -------
        rotation : Int
            Rotation value around the given coordinates.

        References
        ----------
        - GraphicsBase rotate_around_point interfaces
            - https://simon-ritchie.github.io/apysc/en/graphics_base_rotation_around_point.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color=ap.Color("#0af"))
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50
        ... )
        >>> x: ap.Number = ap.Number(100)
        >>> y: ap.Number = ap.Number(100)
        >>> rectangle.set_rotation_around_point(rotation=ap.Int(45), x=x, y=y)
        >>> rectangle.get_rotation_around_point(x=x, y=y)
        Int(45)
        """
        from apysc._display import rotation_interface_helper
        from apysc._type.expression_string import ExpressionString

        self._initialize_rotation_around_point_if_not_initialized()
        default_val: Int = Int(0)
        key_exp_str: ExpressionString = (
            rotation_interface_helper.get_coordinates_key_for_expression(
                x=float(x._value), y=float(y._value)
            )
        )
        rotation: Int = self._rotation_around_point.get(
            key=key_exp_str, default=default_val
        )
        return rotation

    @final
    @arg_validation_decos.is_integer(arg_position_index=1, optional=False)
    @arg_validation_decos.is_num(arg_position_index=2, optional=False)
    @arg_validation_decos.is_num(arg_position_index=3, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def set_rotation_around_point(self, *, rotation: Int, x: Number, y: Number) -> None:
        """
        Update a rotation value around the given coordinates.

        Parameters
        ----------
        rotation : Int
            Rotation value to set.
        x : Number
            X-coordinate.
        y : Number
            Y-coordinate.

        References
        ----------
        - GraphicsBase rotate_around_point interfaces
            - https://simon-ritchie.github.io/apysc/en/graphics_base_rotation_around_point.html  # noqa
        """
        from apysc._display import rotation_interface_helper
        from apysc._type.expression_string import ExpressionString

        self._initialize_rotation_around_point_if_not_initialized()
        key_exp_str: ExpressionString = (
            rotation_interface_helper.get_coordinates_key_for_expression(
                x=int(x._value), y=int(y._value)
            )
        )
        self._rotation_around_point._value[key_exp_str.value] = rotation
        self._append_rotation_around_point_update_expression(
            rotation=rotation, x=x, y=y
        )

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_rotation_around_point_update_expression(
        self, *, rotation: Int, x: Number, y: Number
    ) -> None:
        """
        Append a rotation value around the given coordinates
        updating expression.

        Parameters
        ----------
        rotation : Int
            Rotation value to set.
        x : Number
            X-coordinate.
        y : Number
            Y-coordinate.
        """
        from apysc._expression import expression_data_util

        expression: str = self._get_rotation_around_point_updating_expression(
            rotation=rotation, x=x, y=y
        )
        expression_data_util.append_js_expression(expression=expression)

    @final
    def _get_rotation_around_point_updating_expression(
        self, *, rotation: Int, x: Number, y: Number
    ) -> str:
        """
        Get a rotation value around the given coordinates'
        updating expression string.

        Parameters
        ----------
        rotation : Int
            Rotation value to set.
        x : Number
            X-coordinate.
        y : Number
            Y-coordinate.

        Returns
        -------
        expression : str
            A rotation value around the given coordinates'
            updating expression string.
        """
        from apysc._display import rotation_interface_helper
        from apysc._expression import expression_variables_util
        from apysc._expression import var_names
        from apysc._type import value_util
        from apysc._type.expression_string import ExpressionString

        self._initialize_rotation_around_point_if_not_initialized()
        before_value_str: str = expression_variables_util.get_next_variable_name(
            type_name=var_names.INT
        )
        key_exp_str: ExpressionString = (
            rotation_interface_helper.get_coordinates_key_for_expression(x=x, y=y)
        )
        after_value_str: str = value_util.get_value_str_for_expression(value=rotation)
        x_value_str: str = value_util.get_value_str_for_expression(value=x)
        y_value_str: str = value_util.get_value_str_for_expression(value=y)
        rotation_around_point_value_str: str = value_util.get_value_str_for_expression(
            value=self._rotation_around_point
        )
        expression: str = (
            f"if ({key_exp_str.value} in "
            f"{rotation_around_point_value_str}) {{"
            f"\n  var {before_value_str} = "
            f"{rotation_around_point_value_str}[{key_exp_str.value}];"
            "\n}else {"
            f"\n  {before_value_str} = 0;"
            "\n}"
            f"\n{self.variable_name}.rotate("
            f"-{before_value_str}, {x_value_str}, {y_value_str});"
            f"\n{self.variable_name}.rotate("
            f"{after_value_str}, {x_value_str}, {y_value_str});"
            f"\n{rotation_around_point_value_str}[{key_exp_str.value}] = "
            f"{after_value_str};"
        )
        return expression

    _rotation_around_point_snapshots: Optional[Dict[str, Dict[str, Any]]] = None

    def _make_snapshot(self, *, snapshot_name: str) -> None:
        """
        Make a value snapshot.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._initialize_rotation_around_point_if_not_initialized()
        self._set_single_snapshot_val_to_dict(
            dict_name="_rotation_around_point_snapshots",
            value={**self._rotation_around_point._value},
            snapshot_name=snapshot_name,
        )

    def _revert(self, *, snapshot_name: str) -> None:
        """
        Revert a value if a snapshot exists.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._rotation_around_point._value = self._get_snapshot_val_if_exists(
            current_value=self._rotation_around_point._value,
            snapshot_dict=self._rotation_around_point_snapshots,
            snapshot_name=snapshot_name,
        )



"""Class implementation for mouse event unbingins-related mix-in.
"""

from typing import Callable
from typing import Dict
from typing import TypeVar

from typing_extensions import final

from apysc._event.handler import HandlerData
from apysc._event.mouse_event import MouseEvent
from apysc._event.mouse_event_type import MouseEventType
from apysc._event.set_handler_data_mixin import SetHandlerDataMixIn
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.variable_name_mixin import VariableNameMixIn

_Options = TypeVar("_Options")
_Handler = Callable[[MouseEvent, _Options], None]


class MouseEventUnbindingMixIn(SetHandlerDataMixIn[MouseEvent]):
    @final
    @add_debug_info_setting(module_name=__name__)
    def _unbind_mouse_event(
        self,
        *,
        handler: _Handler[_Options],
        mouse_event_type: MouseEventType,
        handlers_dict: Dict[str, HandlerData[MouseEvent]],
    ) -> None:
        """
        Unbind a specified handler's mouse event.

        Parameters
        ----------
        handler : _Handler
            Unbinding target Callable.
        mouse_event_type : MouseEventType
            Event type to unbind.
        handlers_dict : dict
            Dictionary that has handler's data.
        """
        from apysc._event.handler import append_unbinding_expression
        from apysc._event.handler import get_handler_name
        from apysc._validation.variable_name_validation import (
            validate_variable_name_mixin_type,
        )

        self_instance: VariableNameMixIn = validate_variable_name_mixin_type(
            instance=self
        )
        name: str = get_handler_name(handler=handler, instance=self)
        if name in handlers_dict:
            del handlers_dict[name]
        append_unbinding_expression(
            this=self_instance, handler_name=name, mouse_event_type=mouse_event_type
        )

    @final
    @add_debug_info_setting(module_name=__name__)
    def _unbind_all_mouse_events(
        self,
        *,
        mouse_event_type: MouseEventType,
        handlers_dict: Dict[str, HandlerData[MouseEvent]],
    ) -> None:
        """
        Unbind specified all mouse event type's events.

        Parameters
        ----------
        mouse_event_type : MouseEventType
            Event type to unbind.
        handlers_dict : dict
            Dictionary that has handler's data.
        """
        from apysc._event.handler import append_unbinding_all_expression
        from apysc._validation.variable_name_validation import (
            validate_variable_name_mixin_type,
        )

        self_instance: VariableNameMixIn = validate_variable_name_mixin_type(
            instance=self
        )
        handlers_dict.clear()
        append_unbinding_all_expression(
            this=self_instance, mouse_event_type=mouse_event_type
        )



"""This module is for the translation mapping data of the
following document:

Document file: graphics_base_line_dash_dot_setting.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# GraphicsBase line_dash_dot_setting interface": "# GraphicsBase クラスの line_dash_dot_setting インターフェイス",  # noqa
    ##################################################
    "This page explains the `GraphicsBase` class `line_dash_dot_setting` property interface.": "このページでは`GraphicsBase`クラスの`line_dash_dot_setting`属性のインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `line_dash_dot_setting` property interface updates or get the instance's current line dash-dot setting.": "`line_dash_dot_setting`属性ではインスタンスの現在の一点鎖線の設定の更新もしくは取得を行えます。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The getter or setter interface value becomes (or requires) the `LineDashDotSetting` instance value.": "getterやsetterのインターフェイスの値は`LineDashDotSetting`インスタンスの値となります。",  # noqa
    ##################################################
    "The following example sets the 10-pixel dash and 3-pixel dot to the line:": "以下のコード例では10pxの破線と3pxの点線の一点鎖線を設定しています。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=250,\n    stage_height=100,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.line_style(color=ap.Color("#0af"), thickness=5)\n\nline: ap.Line = sprite.graphics.draw_line(x_start=50, y_start=50, x_end=200, y_end=50)\nline.line_dash_dot_setting = ap.LineDashDotSetting(\n    dot_size=10, dash_size=3, space_size=3\n)\n\nap.save_overall_html(dest_dir_path="./graphics_base_line_dash_dot_setting_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=250,\n    stage_height=100,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.line_style(color=ap.Color("#0af"), thickness=5)\n\nline: ap.Line = sprite.graphics.draw_line(x_start=50, y_start=50, x_end=200, y_end=50)\nline.line_dash_dot_setting = ap.LineDashDotSetting(\n    dot_size=10, dash_size=3, space_size=3\n)\n\nap.save_overall_html(dest_dir_path="./graphics_base_line_dash_dot_setting_basic_usage/")\n```',  # noqa
    ##################################################
    "## Delete setting": "## 設定の削除",
    ##################################################
    "The `delete_line_dash_dot_setting` interface deletes this line setting.": "`delete_line_dash_dot_setting`インターフェイスではこの線の設定を削除します。",  # noqa
    ##################################################
    "In the following example, if you click the rectangle, the handler deletes the line setting:": "以下の例では四角をクリックするとハンドラ内の処理で線の設定を削除しています:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler for the click event.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent[ap.Rectangle]\n        Event instance.\n    options : dict\n        Optional argument dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n    rectangle.delete_line_dash_dot_setting()\n\n\nap.Stage(\n    stage_width=150,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nrectangle: ap.Rectangle = ap.Rectangle(\n    x=50,\n    y=50,\n    width=50,\n    height=50,\n    fill_color=ap.Color("#666"),\n    line_color=ap.Color("#fff"),\n    line_thickness=2,\n    line_dash_dot_setting=ap.LineDashDotSetting(dot_size=2, dash_size=4, space_size=2),\n)\nrectangle.click(handler=on_click)\nap.save_overall_html(\n    dest_dir_path="./graphics_base_line_dash_dot_setting_delete_setting/"\n)\n```': '```py\n# runnable\nimport apysc as ap\n\n\ndef on_click(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler for the click event.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent[ap.Rectangle]\n        Event instance.\n    options : dict\n        Optional argument dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n    rectangle.delete_line_dash_dot_setting()\n\n\nap.Stage(\n    stage_width=150,\n    stage_height=150,\n    background_color=ap.Color("#333"),\n    stage_elem_id="stage",\n)\nrectangle: ap.Rectangle = ap.Rectangle(\n    x=50,\n    y=50,\n    width=50,\n    height=50,\n    fill_color=ap.Color("#666"),\n    line_color=ap.Color("#fff"),\n    line_thickness=2,\n    line_dash_dot_setting=ap.LineDashDotSetting(dot_size=2, dash_size=4, space_size=2),\n)\nrectangle.click(handler=on_click)\nap.save_overall_html(\n    dest_dir_path="./graphics_base_line_dash_dot_setting_delete_setting/"\n)\n```',  # noqa
    ##################################################
    "## line_dash_dot_setting API": "## line_dash_dot_setting のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get a current dash-dot (1-dot chain) setting.<hr>": "現在の一点鎖線のスタイル設定を取得します。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `line_dash_dot_setting`: LineDashDotSetting or None": "- `line_dash_dot_setting`: LineDashDotSetting or None",  # noqa
    ##################################################
    "  - Dash-dot (1-dot chain) setting.": "  - 一点鎖線の設定。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.line_style(color=ap.Color("#fff"), thickness=10)\n>>> line: ap.Line = sprite.graphics.draw_line(\n...     x_start=50, y_start=50, x_end=150, y_end=50\n... )\n>>> line.line_dash_dot_setting = ap.LineDashDotSetting(\n...     dot_size=2, dash_size=5, space_size=3\n... )\n>>> line.line_dash_dot_setting.dot_size\nInt(2)\n\n>>> line.line_dash_dot_setting.dash_size\nInt(5)\n\n>>> line.line_dash_dot_setting.space_size\nInt(3)\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.line_style(color=ap.Color("#fff"), thickness=10)\n>>> line: ap.Line = sprite.graphics.draw_line(\n...     x_start=50, y_start=50, x_end=150, y_end=50\n... )\n>>> line.line_dash_dot_setting = ap.LineDashDotSetting(\n...     dot_size=2, dash_size=5, space_size=3\n... )\n>>> line.line_dash_dot_setting.dot_size\nInt(2)\n\n>>> line.line_dash_dot_setting.dash_size\nInt(5)\n\n>>> line.line_dash_dot_setting.space_size\nInt(3)\n```',  # noqa
    ##################################################
    "## delete_line_dash_dot_setting API": "## delete_line_dash_dot_setting のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Delete a current line dash-dot (1-dot chain) setting.": "現在の線の一点鎖線の設定を削除します。",
}



"""The class implementation for the outlined `phonelink_ring` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialPhonelinkRingOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `phonelink_ring` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M20.1 7.7l-1 1c1.8 1.8 1.8 4.6 0 6.5l1 1c2.5-2.3 2.5-6.1 0-8.5zM18 9.8l-1 1c.5.7.5 1.6 0 2.3l1 1c1.2-1.2 1.2-3 0-4.3zM14 1H4c-1.1 0-2 .9-2 2v18c0 1.1.9 2 2 2h10c1.1 0 2-.9 2-2V3c0-1.1-.9-2-2-2zm0 19H4V4h10v16z"/></svg>'  # noqa



"""The mix-in class implementation for the `ColorCopyMixIn` class.
"""

from copy import deepcopy
from typing import TYPE_CHECKING
from typing import Generic
from typing import TypeVar
from typing import cast

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.copy_interface import CopyInterface

if TYPE_CHECKING:
    from apysc._color.color import Color

_SelfType = TypeVar("_SelfType", bound="Color")


class ColorCopyMixIn(
    CopyInterface,
    Generic[_SelfType],
):
    @final
    @add_debug_info_setting(module_name=__name__)
    def copy(self) -> _SelfType:
        """
        Make a deep copy of this instance.

        Returns
        -------
        result : _SelfType
            Copied instance.
        """
        from apysc._expression import event_handler_scope
        from apysc._expression import expression_data_util
        from apysc._expression import expression_variables_util
        from apysc._expression.event_handler_scope import TemporaryNotHandlerScope

        result: _SelfType = cast(_SelfType, deepcopy(self))
        result._value.variable_name = expression_variables_util.get_next_variable_name(
            type_name=result._value.type_name,
        )
        with TemporaryNotHandlerScope():
            expression: str = self._get_copy_expression(result=result)
            expression_data_util.append_js_expression(expression=expression)

        if event_handler_scope.current_scope_is_in_event_handler():
            expression_data_util.append_js_expression(expression=expression)

        result._variable_name = result._value.variable_name
        return result

    def _get_copy_expression(self, *, result: _SelfType) -> str:
        """
        Get a copy expression.

        Parameters
        ----------
        result : _SelfType
            A copied instance.

        Returns
        -------
        expression : str
            A target expression.
        """
        from apysc._validation import variable_name_validation

        self_value_variable_name: str = (
            variable_name_validation.validate_variable_name_mixin_type(
                instance=getattr(self, "_value"),
            ).variable_name
        )
        expression: str = (
            f"var {result._value.variable_name} = {self_value_variable_name};"
        )
        return expression



"""The class implementation for the outlined `supervised_user_circle` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialSupervisedUserCircleOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `supervised_user_circle` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M12.5 10c0-1.65-1.35-3-3-3s-3 1.35-3 3 1.35 3 3 3 3-1.35 3-3zm-3 1c-.55 0-1-.45-1-1s.45-1 1-1 1 .45 1 1-.45 1-1 1zm6.5 2c1.11 0 2-.89 2-2 0-1.11-.89-2-2-2-1.11 0-2.01.89-2 2 0 1.11.89 2 2 2zM11.99 2.01c-5.52 0-10 4.48-10 10s4.48 10 10 10 10-4.48 10-10-4.48-10-10-10zM5.84 17.12c.68-.54 2.27-1.11 3.66-1.11.07 0 .15.01.23.01.24-.64.67-1.29 1.3-1.86-.56-.1-1.09-.16-1.53-.16-1.3 0-3.39.45-4.73 1.43-.5-1.04-.78-2.2-.78-3.43 0-4.41 3.59-8 8-8s8 3.59 8 8c0 1.2-.27 2.34-.75 3.37-1-.59-2.36-.87-3.24-.87-1.52 0-4.5.81-4.5 2.7v2.78c-2.27-.13-4.29-1.21-5.66-2.86z"/></svg>'  # noqa



"""The loop implementation class for the `ap.Dictionary` keys.
"""

from typing import Any
from typing import Dict
from typing import Generic
from typing import Optional
from typing import Type
from typing import TypeVar
from typing import cast

from typing_extensions import final

from apysc._expression.get_last_scope_interface import GetLastScopeInterface
from apysc._expression.indent_num import Indent
from apysc._expression.last_scope import LastScope
from apysc._html.debug_mode import add_debug_info_setting
from apysc._loop.for_loop_exit_mixin import ForLoopExitMixIn
from apysc._type.boolean import Boolean
from apysc._type.dictionary import Dictionary
from apysc._type.initialize_locals_and_globals_mixin import (
    InitializeLocalsAndGlobalsMixIn,
)
from apysc._type.int import Int
from apysc._type.number import Number
from apysc._type.string import String
from apysc._validation import arg_validation_decos

_DictKey = TypeVar("_DictKey", String, Int, Number, Boolean)


class ForDictKeys(
    ForLoopExitMixIn,
    GetLastScopeInterface,
    InitializeLocalsAndGlobalsMixIn,
    Generic[_DictKey],
):
    """
    The loop implementation class for the `ap.Dictionary` keys.

    References
    ----------
    - ForDictKeys class
        - https://simon-ritchie.github.io/apysc/en/for_dict_keys.html

    Examples
    --------
    >>> import apysc as ap

    >>> dict_: ap.Dictionary[ap.String, ap.Boolean] = ap.Dictionary(
    ...     {
    ...         ap.String("apple"): ap.Boolean(True),
    ...         ap.String("orange"): ap.Boolean(False),
    ...     }
    ... )
    >>> keys: ap.Array[ap.String] = ap.Array([])
    >>> with ap.ForDictKeys(dict_=dict_, dict_key_type=ap.String) as key:
    ...     keys.append(key)
    ...
    >>> ap.assert_arrays_equal(
    ...     keys,
    ...     ["apple", "orange"],
    ... )
    """

    _dict: Dictionary[_DictKey, Any]
    _dict_key_type: Type[_DictKey]
    _variable_name_suffix: str

    @final
    @arg_validation_decos.is_apysc_dict(arg_position_index=1)
    @arg_validation_decos.is_initialize_with_base_value_interface_subclass(
        arg_position_index=2,
    )
    @arg_validation_decos.is_builtin_string(arg_position_index=5, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def __init__(
        self,
        dict_: Dictionary[_DictKey, Any],
        dict_key_type: Type[_DictKey],
        *,
        locals_: Optional[Dict[str, Any]] = None,
        globals_: Optional[Dict[str, Any]] = None,
        variable_name_suffix: str = "",
    ) -> None:
        """
        The loop implementation class for the `ap.Dictionary` keys.

        Parameters
        ----------
        dict_ : Dictionary[_DictKey, Any]
            A dictionary to iterate.
        dict_key_type : Type[_DictKey]
            A dictionary key type. This interface accepts hashable types,
            such as the `String`, `Int`, `Number`, or `Boolean`.
        locals_ : Optional[Dict[str, Any]], optional
            Current scope's local variables. Set locals()
            value to this argument. If specified, this interface
            reverts all local scope VariableNameMixIn
            variables (like Int, Sprite) at the end of a with-statement
            scope. This setting is useful when you don't want to
            update each variable.
        globals_ : Optional[Dict[str, Any]], optional
            Current scope's global variables. Set globals()
            value to this argument. This setting works
            the same way as the locals_ argument.
        variable_name_suffix : str, optional
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript debugging.

        References
        ----------
        - ForDictKeys class
            - https://simon-ritchie.github.io/apysc/en/for_dict_keys.html

        Examples
        --------
        >>> import apysc as ap

        >>> dict_: ap.Dictionary[ap.String, ap.Boolean] = ap.Dictionary(
        ...     {
        ...         ap.String("apple"): ap.Boolean(True),
        ...         ap.String("orange"): ap.Boolean(False),
        ...     }
        ... )
        >>> keys: ap.Array[ap.String] = ap.Array([])
        >>> with ap.ForDictKeys(dict_=dict_, dict_key_type=ap.String) as key:
        ...     keys.append(key)
        ...
        >>> ap.assert_arrays_equal(
        ...     keys,
        ...     ["apple", "orange"],
        ... )
        """
        self._initialize_locals_and_globals(locals_=locals_, globals_=globals_)
        self._dict = dict_
        self._dict_key_type = dict_key_type
        self._variable_name_suffix = variable_name_suffix
        self._indent = Indent()

    @final
    def _get_last_scope(self) -> LastScope:
        """
        Get a target last scope value.

        Returns
        -------
        last_scope : LastScope
            A target last scope.
        """
        return LastScope.FOR_DICT_KEYS

    @final
    @add_debug_info_setting(module_name=__name__)
    def __enter__(self) -> _DictKey:
        """
        The entering method for the beginning of with-statement.

        Returns
        -------
        dict_key : _DictKey
            A dictionary key of iteration.
        """
        from apysc._expression import expression_data_util
        from apysc._loop import loop_count
        from apysc._type import revert_mixin

        loop_count.increment_current_loop_count()
        self._snapshot_name = revert_mixin.make_snapshots_of_each_scope_vars(
            locals_=self._locals, globals_=self._globals
        )
        dict_key: _DictKey = cast(
            _DictKey,
            self._dict_key_type._initialize_with_base_value(),
        )

        expression: str = (
            f"for ({dict_key.variable_name} in {self._dict.variable_name}) {{"
        )
        expression_data_util.append_js_expression(expression=expression)

        self._indent.__enter__()
        return dict_key



"""The class implementation for the outlined `settings_remote` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialSettingsRemoteOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `settings_remote` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0z" fill="none"/><path d="M15 9H9c-.55 0-1 .45-1 1v12c0 .55.45 1 1 1h6c.55 0 1-.45 1-1V10c0-.55-.45-1-1-1zm-1 12h-4V11h4v10z"/><circle cx="12" cy="13" r="1"/><path d="M7.05 6.05l1.41 1.41C9.37 6.56 10.62 6 12 6s2.63.56 3.54 1.46l1.41-1.41C15.68 4.78 13.93 4 12 4s-3.68.78-4.95 2.05zM12 0C8.96 0 6.21 1.23 4.22 3.22l1.41 1.41C7.26 3.01 9.51 2 12 2s4.74 1.01 6.36 2.64l1.41-1.41C17.79 1.23 15.04 0 12 0z"/></svg>'  # noqa



"""This module is for the translation mapping data of the
following document:

Document file: mouseover_and_mouseout.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# mouseover and mouseout interfaces": "# mouseover と mouseout のインターフェイス",
    ##################################################
    "This page explains the `mouseover` and `mouseout` interfaces.": "このページでは`mouseover`と`mouseout`の各インターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interfaces are these?": "## 各インターフェイスの概要",
    ##################################################
    "The `mouseover` interface binds the event handler. Moreover, this interface calls the handler when a mouse cursor is over on a `DisplayObject` instance. Conversely, the `mouseout` interface also binds and calls the handler when a cursor is out from the `DisplayObject` one.": "`mouseover`インターフェイスはマウスカーソルが対象の`DisplayObject`インスタンス上に乗った時のイベントのハンドラを設定します。逆に`mouseout`インターフェイスはマウスカーソルが対象の`DisplayObject`上から離れた時のイベントのハンドラを設定します。",  # noqa
    ##################################################
    "## See also": "## 関連資料",
    ##################################################
    "The following page describes the basic mouse event interfaces.": "以下のページでは基本的なマウスイベントの各インターフェイスについて説明しています。",  # noqa
    ##################################################
    "- [Basic mouse event interfaces](mouse_event_basic.md)": "- [基本的なマウスイベントの各インターフェイス](jp_mouse_event_basic.md)",  # noqa
    ##################################################
    "## Basic usage of the mouseover and mouseout interfaces": "## mouseover と mouseout の各インターフェイスの基本的な使い方",  # noqa
    ##################################################
    "Each `DisplayObject` instance has the `mouseover` and `mouseout` interfaces, and you can bind handlers by these.": "`DisplayObject`の各インスタンスは`mouseover`と`mouseout`の各インターフェイスを持っており、それらを使ってハンドラを設定することができます。",  # noqa
    ##################################################
    "The following example binds the mouse over and handler and mouse out one to the rectangle. The rectangle color changes when your cursor is over the rectangle. Also, it reverts to the original one when your cursor is outed from the rectangle.": "以下のコード例ではマウスが乗った時と離れた時のイベントのハンドラを四角に対して設定しています。四角にマウスカーソルが乗った時に四角の色が変更され、カーソルが離れた時に色が戻されるように設定してあります。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_mouseover(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when mouseover.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n\n    # Change the rectangle fill color to magenta.\n    rectangle.fill_color = ap.Color("#f0a")\n\n\ndef on_mouseout(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when mouseout.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n\n    # Revert the rectangle fill color.\n    rectangle.fill_color = ap.Color("#0af")\n\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=150,\n    stage_height=150,\n    stage_elem_id="stage",\n)\n\nsprite: ap.Sprite = ap.Sprite()\n\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\n# Bind the mouse over and mouse out event handlers to the rectangle.\nrectangle.mouseover(on_mouseover)\nrectangle.mouseout(on_mouseout)\n\nap.save_overall_html(dest_dir_path="mouseover_and_mouseout_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\n\ndef on_mouseover(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when mouseover.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n\n    # Change the rectangle fill color to magenta.\n    rectangle.fill_color = ap.Color("#f0a")\n\n\ndef on_mouseout(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when mouseout.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n\n    # Revert the rectangle fill color.\n    rectangle.fill_color = ap.Color("#0af")\n\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=150,\n    stage_height=150,\n    stage_elem_id="stage",\n)\n\nsprite: ap.Sprite = ap.Sprite()\n\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\n# Bind the mouse over and mouse out event handlers to the rectangle.\nrectangle.mouseover(on_mouseover)\nrectangle.mouseout(on_mouseout)\n\nap.save_overall_html(dest_dir_path="mouseover_and_mouseout_basic_usage/")\n```',  # noqa
    ##################################################
    "## Unbind interfaces": "## 解除用のインターフェイス",
    ##################################################
    "The `unbind_mouseover` and `unbind_mouseout` interfaces unbind each registered handler from the `DisplayObject`\\.": "`unbind_mouseover`と`unbind_mouseout`の各インターフェイスは`DisplayObject`から登録されているハンドラの設定を解除します。",  # noqa
    ##################################################
    "The following example unbind handlers in the `on_mouseover` and `on_mouseout` functions so that the interface calls these handlers only once.": "以下のコード例では`on_mouseover`と`on_mouseout`のハンドラの関数内でハンドラの設定を解除しているためこれらのハンドラは最初の1回のみ呼ばれます。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_mouseover(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when mouseover.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n    rectangle.fill_color = ap.Color("#f0a")\n\n    # Unbind the mouseover handler.\n    rectangle.unbind_mouseover(handler=on_mouseover)\n\n\ndef on_mouseout(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when mouseout.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n    rectangle.fill_color = ap.Color("#0af")\n\n    rectangle.unbind_mouseout(handler=on_mouseout)\n\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=150,\n    stage_height=150,\n    stage_elem_id="stage",\n)\n\nsprite: ap.Sprite = ap.Sprite()\n\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\nrectangle.mouseover(on_mouseover)\nrectangle.mouseout(on_mouseout)\n\nap.save_overall_html(dest_dir_path="mouseover_and_mouseout_unbind_interfaces/")\n```': '```py\n# runnable\nimport apysc as ap\n\n\ndef on_mouseover(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when mouseover.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n    rectangle.fill_color = ap.Color("#f0a")\n\n    # Unbind the mouseover handler.\n    rectangle.unbind_mouseover(handler=on_mouseover)\n\n\ndef on_mouseout(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when mouseout.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n    rectangle.fill_color = ap.Color("#0af")\n\n    rectangle.unbind_mouseout(handler=on_mouseout)\n\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=150,\n    stage_height=150,\n    stage_elem_id="stage",\n)\n\nsprite: ap.Sprite = ap.Sprite()\n\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\nrectangle.mouseover(on_mouseover)\nrectangle.mouseout(on_mouseout)\n\nap.save_overall_html(dest_dir_path="mouseover_and_mouseout_unbind_interfaces/")\n```',  # noqa
    ##################################################
    "There are also existing the `unbind_mouseover_all` and `unbind_mouseout_all` interfaces. These interfaces unbind all the handlers from the target `DisplayObject` instance.": "`unbind_mouseover_all`と`unbind_mouseout_all`の各インターフェイスも存在します。これらのインターフェイスは対象の`DisplayObject`のインスタンスから対象のイベントのハンドラ設定を全て解除します。",  # noqa
    ##################################################
    "## mouseover API": "## mouseover API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Add mouse over event listener setting.<hr>": "マウスカーソルが乗った時のイベントのハンドラ設定を追加します。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `handler`: _Handler": "- `handler`: _Handler",
    ##################################################
    "  - Callable that would be called when mouse over on this instance.": "  - インスタンス上にマウスカーソルが乗った際に呼ばれる関数もしくはメソッド。",  # noqa
    ##################################################
    "- `options`: dict or None, default None": "- `options`: dict or None, default None",  # noqa
    ##################################################
    "  - Optional arguments dictionary to be passed to a handler.": "  - ハンドラに渡される省略が可能な追加のパラメーターとしての辞書。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `name`: str": "- `name`: str",
    ##################################################
    "  - Handler's name.": "  - ハンドラ名。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> def on_mouseover(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.fill_color = ap.Color("#f0a")\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.mouseout(on_mouseover)\n```': '```py\n>>> import apysc as ap\n>>> def on_mouseover(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.fill_color = ap.Color("#f0a")\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.mouseout(on_mouseover)\n```',  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[References]**": "**[関連資料]**",
    ##################################################
    "- [About the handler options' type](https://simon-ritchie.github.io/apysc/en/about_handler_options_type.html)": "- [ハンドラのoptions引数の型について](https://simon-ritchie.github.io/apysc/jp/jp_about_handler_options_type.html)",  # noqa
    ##################################################
    "## unbind_mouseover API": "## unbind_mouseover API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Unbind a specified handler's mouseover event.<hr>": "マウスカーソルが乗った際のイベントの指定されたハンドラ設定を解除します。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `handler`: _Handler": "- `handler`: _Handler",
    ##################################################
    "  - Unbinding target Callable.": "  - イベント設定を取り除く対象の関数やメソッドなど。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> def on_mouseover(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.fill_color = ap.Color("#f0a")\n...     rectangle.unbind_mouseover(on_mouseover)\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.mouseout(on_mouseover)\n```': '```py\n>>> import apysc as ap\n>>> def on_mouseover(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.fill_color = ap.Color("#f0a")\n...     rectangle.unbind_mouseover(on_mouseover)\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.mouseout(on_mouseover)\n```',  # noqa
    ##################################################
    "## unbind_mouseover_all API": "## unbind_mouseover_all API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Unbind all mouseover events.<hr>": "マウスカーソルが乗った際のイベントの全てのハンドラ設定を解除します。<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> def on_mouseover(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.fill_color = ap.Color("#f0a")\n...     rectangle.unbind_mouseover_all()\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.mouseout(on_mouseover)\n```': '```py\n>>> import apysc as ap\n>>> def on_mouseover(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.fill_color = ap.Color("#f0a")\n...     rectangle.unbind_mouseover_all()\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.mouseout(on_mouseover)\n```',  # noqa
    ##################################################
    "## mouseout API": "## mouseout API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Add mouse out event listener setting.<hr>": "マウスカーソルがインスタンス上から離れた際のイベントのハンドラを設定します。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `handler`: _Handler": "- `handler`: _Handler",
    ##################################################
    "  - Callable that would be called when mouse out on this instance.": "  - インスタンス上からマウスカーソルが離れた際のイベントの対象のハンドラ設定を解除します。",  # noqa
    ##################################################
    "- `options`: dict or None, default None": "- `options`: dict or None, default None",  # noqa
    ##################################################
    "  - Optional arguments dictionary to be passed to a handler.": "  - ハンドラに渡される省略が可能な追加のパラメーターとしての辞書。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `name`: str": "- `name`: str",
    ##################################################
    "  - Handler's name.": "  - ハンドラ名。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> def on_mouseout(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.fill_color = ap.Color("#f0a")\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.mouseout(on_mouseout)\n```': '```py\n>>> import apysc as ap\n>>> def on_mouseout(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.fill_color = ap.Color("#f0a")\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.mouseout(on_mouseout)\n```',  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[References]**": "**[関連資料]**",
    ##################################################
    "- [About the handler options' type](https://simon-ritchie.github.io/apysc/en/about_handler_options_type.html)": "- [ハンドラのoptions引数の型について](https://simon-ritchie.github.io/apysc/jp/jp_about_handler_options_type.html)",  # noqa
    ##################################################
    "## unbind_mouseout API": "## unbind_mouseout API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Unbind a specified handler's mouse-out event.<hr>": "インスタンス上からマウスカーソルが離れた際のイベントの対象のハンドラ設定を解除します。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `handler`: _Handler": "- `handler`: _Handler",
    ##################################################
    "  - Unbinding target Callable.": "  - イベント設定を取り除く対象の関数やメソッドなど。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> def on_mouseout(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.fill_color = ap.Color("#f0a")\n...     rectangle.unbind_mouseout(on_mouseout)\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.mouseout(on_mouseout)\n```': '```py\n>>> import apysc as ap\n>>> def on_mouseout(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.fill_color = ap.Color("#f0a")\n...     rectangle.unbind_mouseout(on_mouseout)\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.mouseout(on_mouseout)\n```',  # noqa
    ##################################################
    "## unbind_mouseout_all API": "## unbind_mouseout_all API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Unbind all mouse out events.<hr>": "インスタンス上からマウスカーソルが離れた際のイベントのハンドラ設定を全て解除します。<hr>",  # noqa
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> def on_mouseout(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.fill_color = ap.Color("#f0a")\n...     rectangle.unbind_mouseout_all()\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.mouseout(on_mouseout)\n```': '```py\n>>> import apysc as ap\n>>> def on_mouseout(e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.fill_color = ap.Color("#f0a")\n...     rectangle.unbind_mouseout_all()\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=ap.Color("#0af"))\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.mouseout(on_mouseout)\n```',  # noqa
}



"""The class implementation for the `outgoing_mail` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialOutgoingMailIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `outgoing_mail` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/></g><g><g><path d="M18.5,11c0.17,0,0.34,0.01,0.5,0.03V6.87C19,5.84,18.16,5,17.13,5H3.87C2.84,5,2,5.84,2,6.87v10.26 C2,18.16,2.84,19,3.87,19h9.73C13.22,18.25,13,17.4,13,16.5C13,13.46,15.46,11,18.5,11z M10.4,13L4,9.19V7h0.23l6.18,3.68L16.74,7 H17v2.16L10.4,13z"/><polygon points="19,13 17.59,14.41 19.17,16 15,16 15,18 19.17,18 17.59,19.59 19,21 23,17"/></g></g></svg>'  # noqa



"""The class implementation for the SVG text's singleton instance to make a
text span instance.
"""

from typing import TYPE_CHECKING
from typing import Dict

if TYPE_CHECKING:
    from apysc._display.svg_text import SvgText


class SvgTextSingletonForTextSpan:
    _stage_id_key_svg_texts: Dict[int, "SvgText"] = {}

    @classmethod
    def get_instance(cls) -> "SvgText":
        """
        Get a text instance.

        Returns
        -------
        svg_text : SvgText
            A target text instance.
        """
        from apysc._display.stage import Stage
        from apysc._display.stage import get_stage
        from apysc._display.svg_text import SvgText
        from apysc._type.boolean import Boolean

        stage: Stage = get_stage()
        stage_id: int = id(stage)
        if stage_id in cls._stage_id_key_svg_texts:
            return cls._stage_id_key_svg_texts[stage_id]

        svg_text: SvgText = SvgText(text="")
        svg_text.visible = Boolean(False)
        cls._stage_id_key_svg_texts[stage_id] = svg_text
        return svg_text



"""This module is for the translation mapping data of the
following document:

Document file: display_object.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# DisplayObject class": "# DisplayObject クラス",
    ##################################################
    "This page explains the `DisplayObject` class.": "このページでは`DisplayObject`クラスについて説明します。",  # noqa
    ##################################################
    "## What is the DisplayObject?": "## DisplayObject クラスの概要",
    ##################################################
    "The `DisplayObject` is the apysc base class for each display class, such as  `Sprite`, `Rectangle`, `Circle`, or something else.": "`DisplayObject`クラスは`Sprite`や`Rectangle`などのapyscの表示オブジェクトの基底クラスとなります。",  # noqa
    ##################################################
    "You can verify the `DisplayObject` inheritance with each instance by the `isinstance` function.": "各インスタンスが`DisplayObject`を継承していることを`isinstance`関数を使って確認することができます。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=200,\n    stage_elem_id="stage",\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\ncircle: ap.Circle = sprite.graphics.draw_circle(x=100, y=100, radius=100)\n\n# Verify each instance type.\nassert isinstance(sprite, ap.DisplayObject)\nassert isinstance(circle, ap.DisplayObject)\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=200,\n    stage_height=200,\n    stage_elem_id="stage",\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=ap.Color("#0af"))\ncircle: ap.Circle = sprite.graphics.draw_circle(x=100, y=100, radius=100)\n\n# Verify each instance type.\nassert isinstance(sprite, ap.DisplayObject)\nassert isinstance(circle, ap.DisplayObject)\n```',  # noqa
    ##################################################
    "The apysc uses this class for basic interfaces or creates a new `DisplayObject` instance with the `DisplayObject` inheritance.": "apyscはこのクラスを基本的な共通のインターフェイスで使用したり、もしくは別の`DisplayObject`を継承したインスタンスの作成処理などで使用しています。",  # noqa
    ##################################################
    "The `DisplayObject` class has the basic interfaces, like `x`, `y`, `visible`, each mouse event binding, or others. The following page explains these interfaces one by one.": "`DisplayObject`クラスは`x`や`y`、`visible`属性、各マウスイベントの設定などの基本的なインターフェイスを持っています。以下のページではそれぞれのインターフェイスについて1つ1つ詳しく触れています。",  # noqa
    ##################################################
    "## See also": "## 関連資料",
    ##################################################
    "- [DisplayObject class x and y interfaces](display_object_x_and_y.md)": "- [DisplayObject クラスの x と y インターフェイス](jp_display_object_x_and_y.md)",  # noqa
    ##################################################
    "- [DisplayObject class parent interfaces](display_object_parent.md)": "- [DisplayObjectクラス parent （親要素属性）のインターフェイス](jp_display_object_parent.md)",  # noqa
    ##################################################
    "- [DisplayObject class visible interface](display_object_visible.md)": "- [DisplayObject クラスの visible (表示・非表示) のインターフェイス](jp_display_object_visible.md)",  # noqa
    ##################################################
    "- [DisplayObject class mouse event binding interfaces](display_object_mouse_event.md)": "- [DisplayObject クラスのマウスイベント設定の各インターフェイス](jp_display_object_mouse_event.md)",  # noqa
}



"""The class implementation for the `build` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialBuildIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `build` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path clip-rule="evenodd" d="M0 0h24v24H0z" fill="none"/><path d="M22.7 19l-9.1-9.1c.9-2.3.4-5-1.5-6.9-2-2-5-2.4-7.4-1.3L9 6 6 9 1.6 4.7C.4 7.1.9 10.1 2.9 12.1c1.9 1.9 4.6 2.4 6.9 1.5l9.1 9.1c.4.4 1 .4 1.4 0l2.3-2.3c.5-.4.5-1.1.1-1.4z"/></svg>'  # noqa



"""The class implementation for the `flight_takeoff` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialFlightTakeoffIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the `flight_takeoff` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" enable-background="new 0 0 24 24" height="24" viewBox="0 0 24 24" width="24"><g><rect fill="none" height="24" width="24"/></g><g><g><g><path d="M2.5,19h19v2h-19V19z M22.07,9.64c-0.21-0.8-1.04-1.28-1.84-1.06L14.92,10l-6.9-6.43L6.09,4.08l4.14,7.17l-4.97,1.33 l-1.97-1.54l-1.45,0.39l2.59,4.49c0,0,7.12-1.9,16.57-4.43C21.81,11.26,22.28,10.44,22.07,9.64z"/></g></g></g></svg>'  # noqa



"""This module is for the translation mapping data of the
following document:

Document file: dictionary.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Dictionary class": "# Dictionary クラス",
    ##################################################
    "This page explains the `Dictionary` class.": "このページでは`Dictionary`クラスについて説明します。",
    ##################################################
    "Before reading on, maybe it is helpful to read the following page:": "事前に以下のページを確認しておくと読み進める上で役に立つかもしれません:",  # noqa
    ##################################################
    "- [Why the apysc library doesn't use the Python built-in data type](why_apysc_doesnt_use_python_builtin_data_type.md)": "- [なぜapyscではPythonのビルトインのデータの型を使用していないのか](jp_why_apysc_doesnt_use_python_builtin_data_type.md)",  # noqa
    ##################################################
    "## What is the Dictionary?": "## Dictionaryクラスの概要",
    ##################################################
    "The `Dictionary` class is the apysc dictionary class. It behaves like the Python built-in `dict` value.": "`Dictionary`クラスはapyscの辞書用のクラスです。ごのクラスはPythonビルトインの`dict`の値のように動作します。",  # noqa
    ##################################################
    "## Constructor method": "## コンストラクタメソッド",
    ##################################################
    "The `Dictionary` class constructor method requires a Python built-in `dict` or `Dictionary` value:": "`Dictionary`のコンストラクタではPythonビルトインの`dict`もしくはapyscの`Dictionary`の値を必要とします:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage()\ndict_1: ap.Dictionary = ap.Dictionary({"a": 10})\nassert dict_1 == {"a": 10}\n\ndict_2: ap.Dictionary = ap.Dictionary(dict_1)\nassert dict_1 == dict_2\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage()\ndict_1: ap.Dictionary = ap.Dictionary({"a": 10})\nassert dict_1 == {"a": 10}\n\ndict_2: ap.Dictionary = ap.Dictionary(dict_1)\nassert dict_1 == dict_2\n```',  # noqa
    ##################################################
    "## Value setter interface": "## 値のsetterのインターフェイス",
    ##################################################
    "A `Dictionary` value can be updated by indexing, like the Python built-in `dict` value:": "`Dictionary`の値はPythonのビルトインの`dict`の値と同じようにインデックスを使って更新することができます:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage()\ndict_1: ap.Dictionary = ap.Dictionary({"a": 10})\ndict_1["a"] = 20\nassert dict_1 == {"a": 20}\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage()\ndict_1: ap.Dictionary = ap.Dictionary({"a": 10})\ndict_1["a"] = 20\nassert dict_1 == {"a": 20}\n```',  # noqa
    ##################################################
    "## Value getter interface": "## 値のgetterのインターフェイス",
    ##################################################
    "A `Dictionary` value also can be retrieved by indexing:": "`Dictionary`の値の取得も同様にインデックスを使って行うことができます:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage()\nint_1: ap.Int = ap.Int(10)\ndict_1: ap.Dictionary = ap.Dictionary({"a": int_1})\nint_2: ap.Int = dict_1["a"]\nassert isinstance(int_2, ap.Int)\nassert int_2 == 10\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage()\nint_1: ap.Int = ap.Int(10)\ndict_1: ap.Dictionary = ap.Dictionary({"a": int_1})\nint_2: ap.Int = dict_1["a"]\nassert isinstance(int_2, ap.Int)\nassert int_2 == 10\n```',  # noqa
    ##################################################
    "## Notes of the getter interface": "## getterのインターフェイスの特記事項",
    ##################################################
    "If a `Dictionary` value doesn't have the specified key, a retrieved value type becomes the `AnyValue` type. This behavior occasionally is helpful when a `Dictionary` value is updated dynamically (e.g., updating by the JavaScript event handler).": "もしも`Dictionary`の値が指定されたキーを持たない場合、取り出される値は`AnyValue`型の値となります。この挙動はJavaScript上でのハンドラでの動的な更新処理などを使う際に便利な時があります。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage()\nint_1: ap.Int = ap.Int(10)\ndict_1: ap.Dictionary = ap.Dictionary({"a": int_1})\nretrieved_val: ap.AnyValue = dict_1["b"]\nassert isinstance(retrieved_val, ap.AnyValue)\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage()\nint_1: ap.Int = ap.Int(10)\ndict_1: ap.Dictionary = ap.Dictionary({"a": int_1})\nretrieved_val: ap.AnyValue = dict_1["b"]\nassert isinstance(retrieved_val, ap.AnyValue)\n```',  # noqa
    ##################################################
    "## Value deletion interface": "## 値の削除のインターフェイス",
    ##################################################
    "A `Dictionary` value can be deleted by the `del` statement, as follows:": "`Dictionary`の値は以下のコード例のように`del`ステートメントで削除することができます。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage()\nint_1: ap.Int = ap.Int(10)\ndict_1: ap.Dictionary = ap.Dictionary({"a": int_1})\ndel dict_1["a"]\nassert dict_1 == {}\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage()\nint_1: ap.Int = ap.Int(10)\ndict_1: ap.Dictionary = ap.Dictionary({"a": int_1})\ndel dict_1["a"]\nassert dict_1 == {}\n```',  # noqa
    ##################################################
    "## Dictionary class constructor API": "## Dictionary クラスのコンストラクタのAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Dictionary class for the apysc library.<hr>": "apyscライブラリのための辞書のクラスです。<hr>",
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `value`: dict or Dictionary": "- `value`: dict or Dictionary",
    ##################################################
    "  - Initial dictionary value.": "  - 辞書の初期値。",
    ##################################################
    '- `variable_name_suffix`: str, default ""': '- `variable_name_suffix`: str, default ""',  # noqa
    ##################################################
    "  - A JavaScript variable name suffix string. This setting is sometimes useful for JavaScript debugging.": "  - JavaScript上の変数のサフィックスの設定です。この設定はJavaScriptのデバッグ時に役立つことがあります。",  # noqa
    ##################################################
    "- `skip_init_substitution_expression_appending`: bool, default False": "- `skip_init_substitution_expression_appending`: bool, default False",  # noqa
    ##################################################
    "  - A boolean indicates whether to skip an initial substitution expression or not. This class uses this option internally.": "  - 初期値の代入のコード表現をスキップするかどうかの真偽値です。このオプションはクラス内部の実装で使用されます。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> dictionary: ap.Dictionary = ap.Dictionary({"a": 10})\n>>> dictionary\nDictionary({\'a\': 10})\n\n>>> dictionary["a"]\n10\n\n>>> dictionary["b"] = 20\n>>> dictionary["b"]\n20\n```': '```py\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> dictionary: ap.Dictionary = ap.Dictionary({"a": 10})\n>>> dictionary\nDictionary({\'a\': 10})\n\n>>> dictionary["a"]\n10\n\n>>> dictionary["b"] = 20\n>>> dictionary["b"]\n20\n```',  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[References]**": "**[関連資料]**",
    ##################################################
    "- [Dictionary class generic type settings](https://simon-ritchie.github.io/apysc/en/dictionary_generic.html)": "- [Dictionary クラスのジェネリックの型設定](https://simon-ritchie.github.io/apysc/jp/jp_dictionary_generic.html)",  # noqa
    ##################################################
    "## value attribute API": "## value 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get a current dict value.<hr>": "現在の辞書の値を取得します。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `value`: dict": "- `value`: dict",
    ##################################################
    "  - Current dict value.": "  - 現在の辞書の値。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> dictionary: ap.Dictionary = ap.Dictionary({})\n>>> dictionary.value = {\"a\": 10}\n>>> dictionary.value\n{'a': 10}\n```": "```py\n>>> import apysc as ap\n>>> _ = ap.Stage()\n>>> dictionary: ap.Dictionary = ap.Dictionary({})\n>>> dictionary.value = {\"a\": 10}\n>>> dictionary.value\n{'a': 10}\n```",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[References]**": "**[関連資料]**",
    ##################################################
    "- [apysc fundamental data classes value interface](https://simon-ritchie.github.io/apysc/en/fundamental_data_classes_value_interface.html)": "- [apyscの基本的なデータクラスの value インターフェイス](https://simon-ritchie.github.io/apysc/jp/jp_fundamental_data_classes_value_interface.html)",  # noqa
}



"""The definitions of the material design button constants.
"""

BUTTON_HEIGHT: int = 40

# The outer padding when there is no icon on a button.
NO_ICON_OUTER_PADDING: int = 24

ICON_SIZE: int = 18

ICON_Y: int = (BUTTON_HEIGHT - ICON_SIZE) // 2

# The outer padding when there is an icon on a button.
ICON_OUTER_PADDING_WIDTH: int = 16

# The inner padding between an icon and a label text.
ICON_INNER_PADDING_WIDTH: int = 8



"""This module is for the translation mapping data of the
following document:

Document file: text_font_size.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Text font_size property": "# テキストの font_size 属性",
    ##################################################
    "This page explains the text-related `font_size` property.": "このページではテキスト関係の`font_size`属性について説明します。",  # noqa
    ##################################################
    "## What property is this?": "## 属性の概要",
    ##################################################
    "The `font_size` property updates or gets the instance's font size (text size).": "`font_size`属性ではフォントサイズ（テキストサイズ）の更新もしくは取得を行うことができます。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The getter and setter interfaces' type becomes an `ap.Int` value.": "getterとsetterの各インターフェイスの値の型は`ap.Int`になります。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=350,\n    stage_height=400,\n    stage_elem_id="stage",\n)\nfont_size_16_text: ap.MultiLineText = ap.MultiLineText(\n    text="Example of font-size = 16. Lorem ipsum dolor sit amet, "\n    "consectetur adipiscing elit, sed do eiusmod tempor incididunt "\n    "ut labore et dolore magna aliqua. Ut enim ad minim veniam.",\n    width=300,\n    fill_color=ap.Color("#00aaff"),\n    x=25,\n    y=25,\n)\nfont_size_16_text.font_size = ap.Int(16)\n\nfont_size_32_text: ap.MultiLineText = ap.MultiLineText(\n    text="Example of font-size = 32. Lorem ipsum dolor sit amet, consectetur.",\n    width=300,\n    fill_color=ap.Color("#00aaff"),\n    x=25,\n    y=190,\n)\nfont_size_32_text.font_size = ap.Int(32)\n\nap.save_overall_html(dest_dir_path="./text_font_size_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color=ap.Color("#333"),\n    stage_width=350,\n    stage_height=400,\n    stage_elem_id="stage",\n)\nfont_size_16_text: ap.MultiLineText = ap.MultiLineText(\n    text="Example of font-size = 16. Lorem ipsum dolor sit amet, "\n    "consectetur adipiscing elit, sed do eiusmod tempor incididunt "\n    "ut labore et dolore magna aliqua. Ut enim ad minim veniam.",\n    width=300,\n    fill_color=ap.Color("#00aaff"),\n    x=25,\n    y=25,\n)\nfont_size_16_text.font_size = ap.Int(16)\n\nfont_size_32_text: ap.MultiLineText = ap.MultiLineText(\n    text="Example of font-size = 32. Lorem ipsum dolor sit amet, consectetur.",\n    width=300,\n    fill_color=ap.Color("#00aaff"),\n    x=25,\n    y=190,\n)\nfont_size_32_text.font_size = ap.Int(32)\n\nap.save_overall_html(dest_dir_path="./text_font_size_basic_usage/")\n```',  # noqa
    ##################################################
    "## font_size property API": "## font_size 属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get a text's font size.<hr>": "テキストのフォントサイズを取得します。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `font_size`: Int": "- `font_size`: Int",
    ##################################################
    "  - A text font size.": "  - テキストのフォントサイズ。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage(\n...     background_color=ap.Color("#333"),\n...     stage_width=350,\n...     stage_height=250,\n...     stage_elem_id="stage",\n... )\n>>> text: ap.MultiLineText = ap.MultiLineText(\n...     text="Example of font-size = 32. Lorem ipsum dolor sit amet, "\n...     "consectetur adipiscing elit, sed do eiusmod tempor incididunt.",\n...     width=300,\n...     fill_color=ap.Color("#00aaff"),\n...     x=25,\n...     y=25,\n... )\n>>> text.font_size = ap.Int(32)\n>>> text.font_size\nInt(32)\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage(\n...     background_color=ap.Color("#333"),\n...     stage_width=350,\n...     stage_height=250,\n...     stage_elem_id="stage",\n... )\n>>> text: ap.MultiLineText = ap.MultiLineText(\n...     text="Example of font-size = 32. Lorem ipsum dolor sit amet, "\n...     "consectetur adipiscing elit, sed do eiusmod tempor incididunt.",\n...     width=300,\n...     fill_color=ap.Color("#00aaff"),\n...     x=25,\n...     y=25,\n... )\n>>> text.font_size = ap.Int(32)\n>>> text.font_size\nInt(32)\n```',  # noqa
}



"""Class implementation for the graphics clear method-related mix-in.
"""

from typing import Optional

from typing_extensions import final

from apysc._color.color import Color
from apysc._color.colorless import COLORLESS
from apysc._display.line_dash_dot_setting import LineDashDotSetting
from apysc._display.line_dash_setting import LineDashSetting
from apysc._display.line_dot_setting import LineDotSetting
from apysc._display.line_round_dot_setting import LineRoundDotSetting
from apysc._display.polyline import Polyline
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.int import Int
from apysc._type.number import Number
from apysc._type.string import String


class GraphicsClearMixIn:
    _fill_color: Color
    _fill_alpha: Number
    _line_color: Color
    _line_thickness: Int
    _line_alpha: Number
    _current_line: Optional[Polyline]
    _line_cap: String
    _line_joints: String
    _line_dot_setting: Optional[LineDotSetting]
    _line_dash_setting: Optional[LineDashSetting]
    _line_round_dot_setting: Optional[LineRoundDotSetting]
    _line_dash_dot_setting: Optional[LineDashDotSetting]

    @final
    @add_debug_info_setting(module_name=__name__)
    def clear(self) -> None:
        """
        Clear all graphics and reset fill and line settings.

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color=ap.Color("#0af"))
        >>> _ = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)
        >>> _ = sprite.graphics.draw_rect(x=150, y=50, width=50, height=50)
        >>> sprite.graphics.num_children
        Int(2)

        >>> sprite.graphics.fill_color
        Color("#00aaff")

        >>> sprite.graphics.clear()
        >>> sprite.graphics.num_children
        Int(0)

        >>> sprite.graphics.fill_color
        Color("")

        References
        ----------
        - Graphics clear interface
            - https://simon-ritchie.github.io/apysc/en/graphics_clear.html
        """
        from apysc._display.begin_fill_mixin import BeginFillMixIn
        from apysc._display.child_mixin import ChildMixIn
        from apysc._display.fill_alpha_mixin import FillAlphaMixIn
        from apysc._display.fill_color_mixin import FillColorMixIn
        from apysc._display.line_alpha_mixin import LineAlphaMixIn
        from apysc._display.line_caps import LineCaps
        from apysc._display.line_color_mixin import LineColorMixIn
        from apysc._display.line_joints import LineJoints
        from apysc._display.line_style_mixin import LineStyleMixIn
        from apysc._display.line_thickness_mixin import LineThicknessMixIn

        if isinstance(self, (FillColorMixIn, BeginFillMixIn)):
            self._initialize_fill_color_if_not_initialized()
        self._fill_color = COLORLESS.copy()
        if isinstance(self, (FillAlphaMixIn, BeginFillMixIn)):
            self._initialize_fill_alpha_if_not_initialized()
        self._fill_alpha.value = 1.0
        if isinstance(self, (LineColorMixIn, LineStyleMixIn)):
            self._initialize_line_color_if_not_initialized()
        self._line_color = COLORLESS.copy()
        if isinstance(self, (LineThicknessMixIn, LineStyleMixIn)):
            self._initialize_line_thickness_if_not_initialized()
        self._line_thickness.value = 1
        if isinstance(self, (LineAlphaMixIn, LineStyleMixIn)):
            self._initialize_line_alpha_if_not_initialized()
        self._line_alpha.value = 1.0
        if isinstance(self, ChildMixIn):
            self._initialize_children_if_not_initialized()
            self.remove_children()
        if hasattr(self, "_current_line"):
            self._current_line = None
        if hasattr(self, "_line_cap"):
            self._line_cap = String(LineCaps.BUTT.value)
        if hasattr(self, "_line_joints"):
            self._line_joints = String(LineJoints.MITER.value)
        self._line_dot_setting = None
        self._line_dash_setting = None
        self._line_round_dot_setting = None
        self._line_dash_dot_setting = None



"""The class implementation for the outlined `thumb_up` material icon.
"""

from apysc._display.fixed_html_svg_icon_base import FixedHtmlSvgIconBase


class MaterialThumbUpOutlinedIcon(FixedHtmlSvgIconBase):
    """
    The class implementation for the outlined `thumb_up` material icon.
    """

    def _get_fixed_svg_icon_html(self) -> str:
        """
        Get a fixed SVG icon HTML string.

        Returns
        -------
        fixed_svg_icon_html : str
            Fixed SVG icon HTML string.

        References
        ----------
        - Material icon document
            - https://simon-ritchie.github.io/apysc/en/material_icon.html
        - Material Symbols & Icons, APACHE LICENSE, VERSION 2.0
            - https://fonts.google.com/icons?icon.size=24&icon.color=%23e8eaed
            - https://www.apache.org/licenses/LICENSE-2.0.html
        """
        return '<svg xmlns="http://www.w3.org/2000/svg" height="24" viewBox="0 0 24 24" width="24"><path d="M0 0h24v24H0V0zm0 0h24v24H0V0z" fill="none"/><path d="M9 21h9c.83 0 1.54-.5 1.84-1.22l3.02-7.05c.09-.23.14-.47.14-.73v-2c0-1.1-.9-2-2-2h-6.31l.95-4.57.03-.32c0-.41-.17-.79-.44-1.06L14.17 1 7.58 7.59C7.22 7.95 7 8.45 7 9v10c0 1.1.9 2 2 2zM9 9l4.34-4.34L12 10h9v2l-3 7H9V9zM1 9h4v12H1z"/></svg>'  # noqa
